########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Create/Group.py,v 1.4 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss create group' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Creates a new Group in the 4Suite repository."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.group')
        if options.has_key('container'):
            containerPath = options['container']
        else:
            containerPath = '/ftss/groups'
        try:
            container = repo.fetchResource(containerPath)
            if container.isResourceType(ResourceTypes.ResourceType.CONTAINER):
                container.createGroup(args['groupName'])
                commit = 1
            else:
                print "The resource '%s' is not a container." % container.getPath()
        finally:
            if commit:
                repo.txCommit()
            else:
                repo.txRollback()
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    # just for help message
    defaultContainer = '/ftss/groups'

    cmd = Command.Command('group',
                          'Creates a new Group in the 4Suite repository.',
                          'developers',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('groupName',
                                                                  'The name of the group to create.',
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option('c',
                                                                    'container=PATH',
                                                                    'The container in which to create the group (default: \'%s\')' % defaultContainer),
                                                    ]),
                          fileName = __file__,
                          )
    return cmd

