########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Create/UriReference.py,v 1.3 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss create urireference' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Creates a new URI Reference file in the 4Suite repository.
The URI can be external, but the media type will *not* be determined automatically."""

import sys, os
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):

    base = Uri.OsPathToUri(os.getcwd(),1);
    base += '/'  #Make it a directory
    src = Uri.BASIC_RESOLVER.normalize(args['sourceUri'],base)

    name = args['name']
    imt = options.get('imt','text/plain')
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.urireference')
        try:
            doc = repo.createUriReferenceFile(name,imt,src)
            commit = 1
        finally:
            if commit:
                repo.txCommit()
            else:
                repo.txRollback()

    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('urireference',
                          'Creates a new URI Reference file in the 4Suite repository',
                          '--imt=text/html /testref.html http://4suite.org/',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  "The name (repo path) of the new URI Reference file",
                                                                  str),
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to reference",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=',
                                                                    'The referenced resource\'s Internet Media Type. Default: text/plain.'),
                                                     ]),

                          fileName = __file__
                          )
    return cmd
