########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/User.py,v 1.8 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss fetch user' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Lists information about a User in the 4Suite repository.
Lists the user's password hash, summarizes ACL and group membership."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import AclConstants, ResourceTypes

def Run(options, args):

    try:
        name = args['userName']
        repo = CommandUtil.GetRepository(options,'4ss.fetch.user')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None:
                if user.isResourceType(ResourceTypes.ResourceType.USER):
                    print "User: %s" % user.getPath()
                    print "-"*10
                    print
                    print "Password Hash: %s" % user.getPassword()
                    print "Read ACL: %s" % user.getAcl(AclConstants.READ_ACCESS)
                    print "Write ACL: %s" % user.getAcl(AclConstants.WRITE_ACCESS)
                    print "Groups: [%s]" % ','.join(map(lambda x:x.getName(),user.getGroups()))
                else:
                    print "%s is a group, not a user." % name
            else:
                print "User %s does not exist." % name
        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')


        return



def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('user',
                          'Lists information about a User in the 4Suite repository.',
                          'root',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  'The name of the user for which to fetch info',
                                                                  str),
                                       ],

                          fileName = __file__
                          )
    return cmd
