########################################################################
#
# File Name:            Passwd.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2002 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command sets a 4Suite repository user's password. \
If you have not used the agent or login commands, you will be prompted for \
a username and password of someone who has write access on the resource.""" 

#import os
import sys, sha, getpass
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options, args):
    while 1:
        p1 = getpass.getpass("Enter new password: ")
        p2 = getpass.getpass("Re-enter password: ")
        if p1 == p2:
            password = p1
            break
        else:
            sys.stderr.write("Passwords do not match, please re-enter\n")

    try:
        commit = 0
        name = args['userName']
        repo = CommandUtil.GetRepository(options,'4ss.passwd')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None and user.isResourceType(ResourceTypes.ResourceType.USER):
                user.setPassword(sha.new(password).hexdigest())
                commit = 1
            else:
                print "User %s does not exist." % name

        finally:
            if commit:
                repo.txCommit()
                print "Password for user %s changed." % name
            else:
                repo.txRollback()
    except FtServerClientException, e:
        raise

def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('passwd',
                          'Set the password for a user in the repository',
                          'molson',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  'The name of the user for whom to change the password',
                                                                  str),
                                       ],
                          fileName = __file__,
                          )
    return cmd


