########################################################################
#
# File Name:            Rdf.Complete.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""


__doc__ = """Perform a RDF complete"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import Util


def Run(options, args):

    ns_map = None
    try:
        repo = CommandUtil.GetRepository(options,'4ss.rdf.complete')
        try:
            model = repo.getModel()
            res = model.complete(options.get('subject'),
                                 options.get('predicate'),
                                 options.get('object'),
                                 statementUri = options.get('uri'),
                                 scope = options.get('scope'))

	    ns_map = Util.GetUserNsMappings(repo) #This is only needed for -xml
        finally:
            try:
                repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')
        return

    if options.get('xml'):
        #This get rather conveluted
        from Ft.Rdf.Drivers import Memory
        from Ft.Rdf import Model
        from Ft.Rdf.Serializers import Dom
        from Ft.Xml.Domlette import PrettyPrint
        mem = Memory.DbAdapter('')
        mem.begin()
        mod = Model.Model(mem)
        mod.add(res)
        ser = Dom.Serializer()
        res = ser.serialize(mod, nsMap=ns_map)
        PrettyPrint(res)
    elif options.get('flat-model'):
        print "Statements"
        print "-"*40
        print
        for s in res:
            print s.subject, s.predicate, s.object, s.uri
    else:
        scopes = {}
        for s in res:
            if not scopes.has_key(s.scope):
                scopes[s.scope] = []
            scopes[s.scope].append(s)
        for k, v in scopes.items():
            print 'Scope: ', k
            print "-"*40
            print
            for s in v:
                print (u'%s %s %s %s'%(s.subject, s.predicate,
                                       Display(s.object, OBJ_LEN),
                                       s.uri)).encode('utf-8')
            print


OBJ_LEN = 50
def Display(text, length):
    if len(text) > length:
        result = text[:length] + '...'
    else:
        result = text[:length]
    result = result.replace("\n", "\\n")
    result = result.replace("\r", "\\r")
    return result


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('complete',
                          'Perform and RDF complete',
                          '--subject=urn:uuid1234456',
                          __doc__,
                          function = Run,
                          arguments = [
                                       ],
                          options = Options.Options([Options.Option(None,'subject=','The subject for the complete'),
                                                     Options.Option(None,'predicate=','The predicate for the complete'),
                                                     Options.Option(None,'object=','The object for the complete'),
                                                     Options.Option(None,'uri=','The URI of the statement to fetch'),
                                                     Options.Option(None,'scope=','The scope of statements to fetch.'),
                                                     Options.Option('x', 'xml', 'Generate Serialized RDF in XML format.'),
                                                     Options.Option(None, 'flat-model', 'Do not show the results sorted by scope.'),
                                      ]),
                          fileName = __file__,
                          )
    return cmd


