########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Update/Document.py,v 1.14 2003/03/20 03:50:20 mbrown Exp $
"""
Implementation of '4ss update document' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Updates the content of an existing XML Document in the 4Suite repository."""

import sys
from Ft.Lib import Uri
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

def Run(options, args):
    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.update.document')
        try:
            doc = repo.fetchResource(args['name'])
            doc.setContent(text)
            if options.has_key('imt'):
                doc.setImt(options['imt'])
            if options.has_key('docdef'):
                docDef = doc.fetchResource(options['docdef'])
                doc.setDocumentDefinition(docDef)
            if options.has_key('nodocdef'):
                doc.setDocumentDefinition(None)
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')

    if args['sourceUri'] != '-':
        sourceStream.close()


def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('document',
                          """Update a document.""",
                          '/repo/path/to/dummy.xml dummy.xml',
                          __doc__,
                          function = Run,
                          arguments = [
                                       Arguments.RequiredArgument('name',
                                                                  "The name (repo path) of the existing document in the repository.",
                                                                  str),
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the source from (Use '-' to read it from standard input)",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=','The internet media type to assign to the '\
                                                                    'document.  If not specified it is left unchanged.'),
                                                     Options.ExclusiveOptions([Options.Option(None,
                                                                                              'docdef=',
                                                                                              'The new document definition for this document'),
                                                                               Options.Option(None,
                                                                                              'nodocdef',
                                                                                              'remove the current doc def'),
                                                                               ]),
                                                     ]),
                          fileName = __file__,

                          )
    return cmd
