########################################################################
#
# File Name:            XmlDocument.py
#
"""
XSLT and XPath extensions supporting the 4SS XmlDocument API
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ns import SCORE_NS
import FtssXsltBase
from Ft.Xml.XPath import Conversions
from Ft.Xml.Xslt import XsltElement, ContentInfo, AttributeInfo



def GetDocumentDefinition(context, path=None):
    path = Conversions.StringValue(path)
    base = FtssXsltBase.FetchBaseObject(context.processor, path)
    dd = base.getDocumentDefinition()
    if dd is None: return ""
    return dd.getAbsolutePath()

class SetDocumentDefinitionElement(XsltElement):
    """
    Change the Document Definition of a XML document
    """
    content = None

    legalAttrs = {
        'path' : AttributeInfo.UriReferenceAvt(
        default=None,
        description='The path of the resource to change the IMT of.  Defaults to current.'),
        'docdef' : AttributeInfo.StringAvt(required=1,
                                           description='The new Document Definition'),
        }

    def instantiate(self, context, processor):
        context.setProcessState(self)
        print "IN HERE!!!!"
        # This may return None (the 'default' from AttributeInfo)
        path          = self._path.evaluate(context)
        dd          = self._docdef.evaluate(context)
        
        obj = FtssXsltBase.FetchBaseObject(processor, path)
        if dd == "":
            obj.setDocumentDefinition(None)
        else:
            docdef = obj.fetchResource(dd);
            obj.setDocumentDefinition(docdef)

        return (context,)



ExtFunctions = {
    (SCORE_NS, 'get-document-definition'): GetDocumentDefinition,    
}

ExtElements = {
#    (NS, 'repo-text-reindex'): TextReindexRepositoryElement,
    (SCORE_NS, 'set-document-definition'): SetDocumentDefinitionElement,    
}
