<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet version="1.0" 
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
>

  <xsl:output method='html' encoding='iso-8859-1'/>

  <!-- Main ==============================================================-->
  
  <xsl:template match="module | package">
    <html>
      <head>
        <title><xsl:value-of select="@name"/></title>
        <style>
          <xsl:comment>
            <xi:include xmlns:xi="http://www.w3.org/2001/XInclude"
                        href="modules.css" parse="text"/>
          </xsl:comment>
        </style>
        <link rel="stylesheet" type="text/css" href="modules.css"/>
      </head>
      <body>
        <h1>
          <xsl:value-of select="@name" />
        </h1>
        <xsl:apply-templates/>
      </body>
    </html>
  </xsl:template>
    
  <xsl:template match="description">
    <xsl:call-template name="format-desc">
      <xsl:with-param name="text">
        <xsl:apply-templates/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>


  <xsl:template match='package' mode='link'>
  </xsl:template>

  <xsl:template match='module' mode='link'>
  </xsl:template>

  <xsl:template match='base' mode='link'>
  </xsl:template>

  <!-- Function Arguments ================================================-->

  <xsl:template match='arguments'>
    <xsl:text>(</xsl:text>
    <xsl:apply-templates select='*' mode='argument'/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match='arg' mode='argument'>
    <xsl:if test='position() &gt; 1'>, </xsl:if>
    <xsl:value-of select='@name'/>
    <xsl:if test='@default'>
      <span class='argument-default'>
        <xsl:text>=</xsl:text>
        <xsl:value-of select='@default'/>
      </span>
    </xsl:if>
  </xsl:template>

  <xsl:template match='sequence' mode='argument'>
    <xsl:if test='position() &gt; 1'>, </xsl:if>
    <xsl:text>(</xsl:text>
    <xsl:apply-templates select='*' mode='argument'/>
    <xsl:if test='count(*) = 1'>
      <!-- add the trailing comma for tuples of length one -->
      <xsl:text>,</xsl:text>
    </xsl:if>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match='var-args' mode='argument'>
    <xsl:if test='position() &gt; 1'>, </xsl:if>
    <xsl:text>*</xsl:text>
    <xsl:value-of select='@name'/>
  </xsl:template>

  <xsl:template match='var-keywords' mode='argument'>
    <xsl:if test='position() &gt; 1'>, </xsl:if>
    <xsl:text>**</xsl:text>
    <xsl:value-of select='@name'/>
  </xsl:template>

  <xsl:template match='unknown' mode='argument'>
    <xsl:text>...</xsl:text>
  </xsl:template>
  
  

  <!-- Multi-column Listing ==============================================-->

  <xsl:template name='multi-column'>
    <xsl:param name='contents' select='/..'/>
    <table width='100%' class='list'>
      <xsl:variable name='rows' select='ceiling(count($contents) div 4)'/>
      <xsl:variable name='row2' select='$rows'/>
      <xsl:variable name='row3' select='$rows * 2'/>
      <xsl:variable name='row4' select='$rows * 3'/>
      <xsl:for-each select='$contents[position() &lt;= $rows]'>
        <xsl:variable name='pos' select='position()'/>
        <tr>
          <td width='25%'>
            <xsl:apply-templates select='$contents[$pos]' mode='multi-column'/>
            <xsl:text>&#160;</xsl:text>
          </td>
          <td width='25%'>
            <xsl:apply-templates select='$contents[$pos + $row2]' mode='multi-column'/>
            <xsl:text>&#160;</xsl:text>
          </td>
          <td width='25%'>
            <xsl:apply-templates select='$contents[$pos + $row3]' mode='multi-column'/>
            <xsl:text>&#160;</xsl:text>
          </td>
          <td width='25%'>
            <xsl:apply-templates select='$contents[$pos + $row4]' mode='multi-column'/>
            <xsl:text>&#160;</xsl:text>
          </td>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

  <xsl:template match="module" mode='multi-column'>
    <xsl:variable name='href'>
      <xsl:choose>
        <xsl:when test='@realname'>
          <xsl:value-of select='@realname'/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:if test='@path'>
            <xsl:value-of select='@path'/>
            <xsl:text>.</xsl:text>
          </xsl:if>
          <xsl:value-of select='@name'/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test='@documented = "no"'>
        <span class='undocumented'>
          <xsl:value-of select='@name'/>
        </span>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name='new-href'>
          <xsl:call-template name='translate-href'>
            <xsl:with-param name='href' select='$href'/>
          </xsl:call-template>
        </xsl:variable>
        <a href='{string($new-href)}'>
          <xsl:value-of select='@name'/>
        </a>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="package" mode='multi-column'>
    <xsl:variable name='href'>
      <xsl:if test='@path'>
        <xsl:value-of select='@path'/>
        <xsl:text>.</xsl:text>
      </xsl:if>
      <xsl:value-of select='@name'/>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test='@documented = "no"'>
        <span class='undocumented'>
          <strong>
            <xsl:value-of select='@name'/>
          </strong>
          <xsl:text>&#160;(package)</xsl:text>
        </span>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name='new-href'>
          <xsl:call-template name='translate-href'>
            <xsl:with-param name='href' select='$href'/>
          </xsl:call-template>
        </xsl:variable>
        <a href='{string($new-href)}'>
          <strong>
            <xsl:value-of select='@name'/>
          </strong>
          <xsl:text>&#160;(package)</xsl:text>
        </a>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- Package Contents ==================================================-->

  <xsl:template match="package-index">
    <!-- children are either 'module' or 'package' -->
    <xsl:variable name='contents' select='*[@public = "yes"]'/>
    <xsl:if test="count($contents)">
      <h2>Package Contents</h2>
      <xsl:call-template name='multi-column'>
      	<xsl:with-param name='contents' select='$contents'/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>


  <!-- Modules ===========================================================-->

  <xsl:template match="modules">
    <xsl:if test="count(module[@public = 'yes'])">
      <h2>Modules</h2>
      <xsl:call-template name='multi-column'>
      	<xsl:with-param name='contents' select="*[@public = 'yes']"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>


  <!-- Classes ===========================================================-->
  
  <xsl:template match="classes">
    <xsl:if test="count(class[@public = 'yes'])">
      <h2>Classes</h2>
      <xsl:apply-templates/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="class">
    <xsl:if test="@public = 'yes'">
      <div class='section'>
        <a name="{@name}"/>
        <dl>
          <dt>
            <div class="class">
              <xsl:text>class </xsl:text>
              <strong><xsl:value-of select="@name"/></strong>
              <xsl:if test='base'>
                <xsl:text>(</xsl:text>
                <xsl:apply-templates select='base' mode='bases'/>
                <xsl:text>)</xsl:text>
              </xsl:if>
            </div>
          </dt>
          <dd>
            <xsl:call-template name='format-desc'> 
              <xsl:with-param name='text' select='string(description)'/>
            </xsl:call-template>
          </dd>
        </dl>
        <xsl:apply-templates select='*' mode='class'/>
      </div>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match='base' mode='bases'>
    <xsl:choose>
      <xsl:when test='@documented = "no"'>
        <span class='undocumented'>
          <xsl:value-of select="@module"/>.<xsl:value-of select="@name"/>
        </span>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name='new-href'>
          <xsl:call-template name='translate-href'>
            <xsl:with-param name='href' select='@module'/>
          </xsl:call-template>
        </xsl:variable>
        <a href='{string($new-href)}#{@name}'>
          <xsl:value-of select="@module"/>.<xsl:value-of select="@name"/>
        </a>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="position() &gt; 1 or position() &lt; last()">, </xsl:if>
  </xsl:template>

  <xsl:template match='*' mode='class'/>

  <xsl:template match='methods' mode='class'>
    <xsl:choose>
      <xsl:when test='@inherited'>
      </xsl:when>
      <xsl:otherwise>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates mode='class'/>
    <xsl:if test='position() != last()'>
      <hr/>
    </xsl:if>
  </xsl:template>

  <xsl:template match='method | class-method | static-method' mode='class'>
    <div class='method'>
      <dl>
        <dt>
          <a name="{@id}">
            <strong>
              <xsl:value-of select='@name'/>
            </strong>
          </a>
          <xsl:if test='@realname'> = </xsl:if>
          <xsl:choose>
            <xsl:when test='@realid'>
              <a href='#{@realid}'>
                <xsl:value-of select='@realname'/>
              </a>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select='@realname'/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:apply-templates select='arguments'/>
        </dt>
        <dd>
          <xsl:call-template name='format-desc'>
            <xsl:with-param name='text' select='string(description)'/>
          </xsl:call-template>
        </dd>
      </dl>
    </div>
  </xsl:template>

  <xsl:template match='properties' mode='class'>
    <xsl:apply-templates mode='class'/>
    <xsl:choose>
      <xsl:when test='@inherited'>
      </xsl:when>
      <xsl:otherwise>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test='position() != last()'>
      <hr/>
    </xsl:if>
  </xsl:template>

  <xsl:template match='property' mode='class'>
    <div class='property'>
      <dl>
        <dt>
          <a name="{@id}">
            <strong>
              <xsl:value-of select='@name'/>
            </strong>
          </a>
          <!-- FIXME: not to implement
          <xsl:if test='@realname'> = </xsl:if>
          <xsl:choose>
            <xsl:when test='@realid'>
              <a href='#{@realid}'>
                <xsl:value-of select='@realname'/>
              </a>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select='@realname'/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>(</xsl:text>
          <xsl:value-of select='@argspec'/>
          <xsl:text>)</xsl:text>
          -->
        </dt>
        <dd>
          <xsl:call-template name='format-desc'>  
            <xsl:with-param name='text' select='string(description)'/>  
          </xsl:call-template>  
        </dd>
      </dl>
    </div>
  </xsl:template>

  <xsl:template match='others' mode='class'>
    <p>
      <xsl:text>Data and other non-method functions </xsl:text>
      <xsl:choose>
        <xsl:when test='@inherited'>
        </xsl:when>
        <xsl:otherwise>defined here:</xsl:otherwise>
      </xsl:choose>
    </p>
    <xsl:apply-templates mode='class'/>
    <xsl:if test='position() != last()'>
      <hr/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match='other' mode='class'>
    <dl>
      <dt>
        <strong><xsl:value-of select="@name"/></strong>
        <xsl:text> = </xsl:text>
        <xsl:value-of select='value'/>
      </dt>
      <dd>
        <xsl:apply-templates select='description'/>
      </dd>
    </dl>
  </xsl:template>


  <!-- Functions ==============================================================-->
  
  <xsl:template match="functions">
    <xsl:variable name='public' select="*[@public = 'yes']"/>
    <xsl:if test="count($public)">
      <h2>Functions</h2>
      <xsl:apply-templates select='$public'/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="function">
    <div class="function">
      <dl>
        <dt>
          <a name="{@id}">
            <strong>
              <xsl:value-of select='@name'/>
            </strong>
          </a>
          <xsl:if test='@realname'> = </xsl:if>
          <xsl:choose>
            <xsl:when test='@realid'>
              <a href='#{@realid}'>
                <xsl:value-of select='@realname'/>
              </a>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select='@realname'/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:apply-templates select='arguments'/>
        </dt>
        <dd>
          <xsl:call-template name='format-desc'>  
            <xsl:with-param name='text' select='string(description)'/>  
          </xsl:call-template>  
        </dd>
      </dl>
    </div>
  </xsl:template>


  <!-- Others ==============================================================-->
  
  <xsl:template match="others">
    <xsl:variable name='public' select="*[@public = 'yes']"/>
    <xsl:if test="count($public)">
      <h2>Data</h2>
      <xsl:apply-templates select='$public'/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="other">
    <dl>
      <dt>
        <strong><xsl:value-of select="@name"/></strong>
        <xsl:text> = </xsl:text>
        <xsl:value-of select='value'/>
      </dt>
      <dd>
        <xsl:apply-templates select='description'/>
      </dd>
    </dl>
  </xsl:template>

  <!--
      Class and method description formatter:

      If only one paragraph, whitespace is normalized and uses desc0 CSS class.

      If multiple paragraphs, first paragraph has whitespace normalized and
         uses desc0 CSS class; remaining paragraphs use desc CSS class and
         have their linefeeds converted to <br>s and leading spaces on each
         line are converted to no-break spaces.

      The first paragraph is identified by 2 consecutive linefeed characters at
      its end.
  -->
  <xsl:template name="format-desc">
    <xsl:param name="text"/>
    <xsl:variable name="first-para" select="normalize-space(substring-before($text,'&#xA;&#xA;'))"/>
    <xsl:variable name="remaining-text" select="substring-after($text,'&#xA;&#xA;')"/>
    <xsl:choose>
      <xsl:when test="normalize-space($remaining-text) and not($first-para)">
        <div class="desc0">
          <xsl:value-of select="normalize-space($remaining-text)"/>
        </div>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="normalize-space($first-para)">
          <div class="desc0">
            <xsl:value-of select="normalize-space($first-para)"/>
          </div>
        </xsl:if>
        <xsl:if test="normalize-space($remaining-text)">
          <div class="desc">
            <br/>
            <xsl:call-template name="lf2br">
              <xsl:with-param name="text" select="$remaining-text"/>
            </xsl:call-template>
          </div>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="lf2br">
    <xsl:param name="text"/>
    <xsl:variable name="txt">
      <xsl:call-template name="ls2nbs">
        <xsl:with-param name="text" select="$text"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="txt-s" select="string($txt)"/>
    <xsl:choose>
      <xsl:when test="contains($txt-s,'&#xA;')">
        <xsl:value-of select="substring-before($txt-s,'&#xA;')"/>
        <br/>
        <xsl:call-template name="lf2br">
          <xsl:with-param name="text" select="substring-after($txt-s,'&#xA;')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$txt-s"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="ls2nbs">
    <xsl:param name="text"/>
    <xsl:choose>
      <xsl:when test="starts-with($text,' ')">
        <xsl:text>&#160;</xsl:text>
        <xsl:call-template name="ls2nbs">
          <xsl:with-param name="text" select="substring-after($text,' ')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- Helper functions to allow overriding of link creation -->
  <xsl:template name='translate-href'>
    <xsl:param name='href'/>
    <xsl:value-of select='concat($href,".html")'/>
  </xsl:template>


  
</xsl:stylesheet>
