'''An introduction to 4RDF'''

import sys
import xml.dom.ext
from xml.dom.ext.reader import PyExpat
reader = PyExpat.Reader()

def MemoryDemo():
    #With the memory driver, the model is not persistent
    from Ft.Rdf.Drivers import Memory
    #Note that the name is not significant for the memory driver
    db = Memory.CreateDb('mydemo')
    db.begin()

    from Ft.Rdf import Model
    #Create a model
    m = Model.Model(db)

    #The model is empty, as we illustrate
    print m.complete(None, None, None)
    
    #Read an RDF serialization into the model from DOM
    doc = reader.fromUri('w3c1.rdf')

    from Ft.Rdf.Serializers.Dom import Serializer
    serializer = Serializer()
    
    serializer.deserialize(m, doc, '')

    #Now we have goodies.  See!
    print m.complete(None, None, None)
    
    #Change the language to french
    from Ft.Rdf import Statement
    #First check the status
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)
    
    ns = Statement.Statement('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', 'en')
    m.remove(ns)
    #Check again
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)

    ns = Statement.Statement('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', 'fr')
    m.add(ns)
    #Check again
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)

    #Now write it all back out in XML
    outdoc = serializer.serialize(m)
    xml.dom.ext.PrettyPrint(outdoc)

    db.commit()
    return


#Dbm driver for simple persistence
def DbmDemo():
    from Ft.Rdf.Drivers import Dbm
    #The dbm database name is the name of the DB or DBM file
    if not Dbm.ExistsDb('mydemo.dbm'):
        db = Dbm.CreateDb('mydemo.dbm')
    else:
        db = Dbm.GetDb('mydemo.dbm')

    db.begin()
    from Ft.Rdf import Model
    #Create a handle to the persistent model
    m = Model.Model(db)

    #The model is empty, as we illustrate
    print m.complete(None, None, None)
    
    #Read an RDF serialization into the model from DOM
    doc = reader.fromUri('w3c1.rdf')


    from Ft.Rdf.Serializers.Dom import Serializer
    serializer = Serializer()
    
    serializer.deserialize(m, doc, '')

    #Now we have goodies.  See!
    print m.complete(None, None, None)
    
    #Change the language to french
    from Ft.Rdf import Statement
    #First check the status
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)
    
    ns = Statement.Statement('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', 'en')
    m.remove(ns)
    #Check again
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)

    ns = Statement.Statement('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', 'fr')
    m.add(ns)
    #Check again
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)

    #Now write it all back out in XML
    outdoc = serializer.serialize(m)
    xml.dom.ext.PrettyPrint(outdoc)

    #Make changes permanent
    db.commit()
    return


def Demo(driver, connect, modelName):
    try:
        driver = "Ft.Rdf.Drivers." + driver
        driver_mod = __import__(driver, {}, {}, ["*"])
    except:
        raise SystemExit("Unknown driver '%s'" % driver)

    #Set up to use the Postgres database of the given instance name
    db = driver_mod.GetDb()

    db.begin()
    from Ft.Rdf import Model
    #Create a handle to the persistent model
    m = Model.Model(db)

    #The model is empty, as we illustrate
    print m.complete(None, None, None)
    
    #Read an RDF serialization into the model from DOM
    doc = reader.fromUri('w3c1.rdf')

    from Ft.Rdf.Serializers.Dom import Serializer
    serializer = Serializer()
    
    serializer.deserialize(m, doc, '')

    #Now we have goodies.  See!
    print m.complete(None, None, None)
    
    #Change the language to french
    from Ft.Rdf import Statement
    #First check the status
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)
    
    ns = Statement.Statement('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', 'en')
    m.remove(ns)
    #Check again
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)

    ns = Statement.Statement('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', 'fr')
    m.add(ns)
    #Check again
    print m.complete('http://www.dlib.org', 'http://purl.org/metadata/dublin_core#Language', None)

    #Now write it all back out in XML
    outdoc = serializer.serialize(m)
    xml.dom.ext.PrettyPrint(outdoc)

    #Make changes permanent
    db.commit()
    return


if __name__ == '__main__':
    print "Run the demo using the memory driver"
    MemoryDemo()
    print "Demo completed\n"
    DbmDemo()
    print "Demo completed\n"
    try:
        print "Attempt to run the demo using the PsycoPG driver"
        from Ft.Rdf.Drivers import Psyco
        #Create the database
        Psyco.CreateDb('dbname=ftrdf_demo', 'demo')
        Demo('Psyco', 'dbname=ftrdf_demo', 'demo')
        print "Demo completed\n"
    except:
        print "Psyco demo could not be completed.\n"
    sys.exit(0)
    try:
        print "Attempt to run the demo using the Postgres driver"
        from Ft.Rdf.Drivers import Postgres
        #Create the database
        Postgres.CreateDb('mydemo')
        Demo('Postgres', 'ftrdf_demo', 'demo')
        print "Demo completed\n"
    except:
        print "Postgres demo could not be completed.\n"

