<?xml version="1.0"?>
<!DOCTYPE xsl:stylesheet [
  <!ENTITY nbsp "&#160;">
  <!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#">
  <!ENTITY f	"http://xmlns.4suite.org/ext">
  <!ENTITY fres "http://xmlns.4suite.org/reserved">
  <!ENTITY dc "http://purl.org/dc/elements/1.1/">
  <!ENTITY fschema "http://schemas.4suite.org/4ss">
  <!ENTITY vcard "http://4suite.org/nexus/rdfs/vcard#">
]>
<xsl:stylesheet
  version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:fcore="http://xmlns.4suite.org/4ss/score" 
  xmlns:frdf="http://xmlns.4suite.org/4ss/rdf" 
  xmlns:fhttp="http://xmlns.4suite.org/4ss/http"
  xmlns:exslt="http://exslt.org/common"
  xmlns:exslt-set="http://exslt.org/sets"
  xmlns:exslt-math="http://exslt.org/math"
  xmlns:exslt-functions="http://exslt.org/functions"
  xmlns:exslt-date-time="http://exslt.org/dates-and-times"
  xmlns:dc="&dc;" 
  xmlns:fres="&fres;"
  xmlns:rdf="&rdf;" 
  xmlns:f="&f;" 
  xmlns:vcard="&vcard;" 
  xmlns:fschema="http://schemas.4suite.org/4ss"
  extension-element-prefixes="f frdf fhttp fcore exslt exslt-set exslt-math exslt-functions exslt-date-time">

  <!--
File:  FILENAME HERE

Description:

    DESCRIPTION HERE


WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information

  -->

  <xsl:template name='count-non-dealer'>
    <xsl:param name='cur-dealer'/>
    <!--
         Count the non dealers hand first
         -->

    <xsl:variable name='non-dealer' select='number($cur-dealer = 1) + 1'/>

    <xsl:variable name='cards'>
      <xsl:for-each select='/cribbage/player-info/player[@ID=number($non-dealer)]/cards/Card'>
        <xsl:copy-of select='.'/>
      </xsl:for-each>
      <xsl:copy-of select='/cribbage/game-info/cut-card/Card'/>
    </xsl:variable>

    <!-- count the non dealer's hand -->
    
    <xsl:variable name='count-res'>
      <xsl:call-template name='count-hand'>
        <xsl:with-param name='cards' select='exslt:node-set($cards)/Card'/>
        <xsl:with-param name='draw' select='/cribbage/game-info/cut-card/Card'/>
        <xsl:with-param name='is-crib' select='0'/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='number($non-dealer)'/>
      <xsl:with-param name='message' select='"Hand:"'/>
    </xsl:call-template>

    <xsl:apply-templates select='exslt:node-set($count-res)/CribbageCount/*' mode='award-points'>
      <xsl:with-param name='player' select='number($non-dealer)'/>
    </xsl:apply-templates>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='number($non-dealer)'/>
      <xsl:with-param name='message' select='concat("Total: ",string(exslt:node-set($count-res)/CribbageCount/@total))'/>
    </xsl:call-template>


  </xsl:template>



  <xsl:template name='count-dealer'>
    <xsl:param name='cur-dealer'/>

    <xsl:variable name='dealer-cards'>
      <xsl:for-each select='/cribbage/player-info/player[@ID=number($cur-dealer)]/cards/Card'>
        <xsl:copy-of select='.'/>
      </xsl:for-each>
      <xsl:copy-of select='/cribbage/game-info/cut-card/Card'/>
    </xsl:variable>

    
    <xsl:variable name='dealer-count-res'>
      <xsl:call-template name='count-hand'>
        <xsl:with-param name='cards' select='exslt:node-set($dealer-cards)/Card'/>
        <xsl:with-param name='draw' select='/cribbage/game-info/cut-card/Card'/>
        <xsl:with-param name='is-crib' select='0'/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='number($cur-dealer)'/>
      <xsl:with-param name='message' select='"Hand:"'/>
    </xsl:call-template>

    <xsl:apply-templates select='exslt:node-set($dealer-count-res)/CribbageCount/*' mode='award-points'>
      <xsl:with-param name='player' select='number($cur-dealer)'/>
    </xsl:apply-templates>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='number($cur-dealer)'/>
      <xsl:with-param name='message' select='concat("Total: ",string(exslt:node-set($dealer-count-res)/CribbageCount/@total))'/>
    </xsl:call-template>

        
    <!-- count the dealer's crib -->
    <xsl:variable name='dealer-crib-cards'>
      <xsl:for-each select='/cribbage/crib-info/Card'>
        <xsl:copy-of select='.'/>
      </xsl:for-each>
      <xsl:copy-of select='/cribbage/game-info/cut-card/Card'/>
    </xsl:variable>

    
    <xsl:variable name='dealer-crib-count-res'>
      <xsl:call-template name='count-hand'>
        <xsl:with-param name='cards' select='exslt:node-set($dealer-crib-cards)/Card'/>
        <xsl:with-param name='draw' select='/cribbage/game-info/cut-card/Card'/>
        <xsl:with-param name='is-crib' select='1'/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='number($cur-dealer)'/>
      <xsl:with-param name='message' select='"Crib:"'/>
    </xsl:call-template>
        
    <xsl:apply-templates select='exslt:node-set($dealer-crib-count-res)/CribbageCount/*' mode='award-points'>
      <xsl:with-param name='player' select='number($cur-dealer)'/>
    </xsl:apply-templates>

    <xsl:call-template name='log-message'>
      <xsl:with-param name='player' select='number($cur-dealer)'/>
      <xsl:with-param name='message' select='concat("Total: ",string(exslt:node-set($dealer-crib-count-res)/CribbageCount/@total))'/>
    </xsl:call-template>

  </xsl:template>

</xsl:stylesheet>