import urllib, os, types
from Ft.Lib import Uri


# Test cases for BaseJoin()
# (base, relative, expected)
basejoin_test_cases = [
    ('','file:docs/xml/4XLink.api','file:docs/xml/4XLink.api'),
    ('file:docs/xml/','file:docs/xml/4XLink.api','file:docs/xml/4XLink.api'),
    ('file:/usr/local/lib/xslt/bar.xslt','http://4Suite.org/foo.xslt','http://4Suite.org/foo.xslt'),
    ]

# Test cases for absolutize();
# all taken from RFC 2396
CURRENT_DOC_URI = 'http://spam.com/bacon/eggs.xml'
BASE_URI = 'http://a/b/c/d;p?q'
# (ref, base, expected)
absolutize_test_cases = [
    ('g:h'     , BASE_URI, 'g:h'),
    ('g'       , BASE_URI, 'http://a/b/c/g'),
    ('./g'     , BASE_URI, 'http://a/b/c/g'),
    ('g/'      , BASE_URI, 'http://a/b/c/g/'),
    ('/g'      , BASE_URI, 'http://a/g'),
    ('//g'     , BASE_URI, 'http://g'),
    ('?y'      , BASE_URI, 'http://a/b/c/?y'),
    ('g?y'     , BASE_URI, 'http://a/b/c/g?y'),
    ('#s'      , BASE_URI, CURRENT_DOC_URI + '#s'),
    ('g#s'     , BASE_URI, 'http://a/b/c/g#s'),
    ('g?y#s'   , BASE_URI, 'http://a/b/c/g?y#s'),
    (';x'      , BASE_URI, 'http://a/b/c/;x'),
    ('g;x'     , BASE_URI, 'http://a/b/c/g;x'),
    ('g;x?y#s' , BASE_URI, 'http://a/b/c/g;x?y#s'),
    ('.'       , BASE_URI, 'http://a/b/c/'),
    ('./'      , BASE_URI, 'http://a/b/c/'),
    ('..'      , BASE_URI, 'http://a/b/'),
    ('../'     , BASE_URI, 'http://a/b/'),
    ('../g'    , BASE_URI, 'http://a/b/g'),
    ('../..'   , BASE_URI, 'http://a/'),
    ('../../'  , BASE_URI, 'http://a/'),
    ('../../g' , BASE_URI, 'http://a/g'),
    (''        , BASE_URI, CURRENT_DOC_URI),
    ('../../../g', BASE_URI, ('http://a/../g', 'http://a/g')),
    ('../../../../g', BASE_URI, ('http://a/../../g', 'http://a/g')),
    ('/./g', BASE_URI, 'http://a/./g'),
    ('/../g', BASE_URI, 'http://a/../g'),
    ('g.', BASE_URI, 'http://a/b/c/g.'),
    ('.g', BASE_URI, 'http://a/b/c/.g'),
    ('g..', BASE_URI, 'http://a/b/c/g..'),
    ('..g', BASE_URI, 'http://a/b/c/..g'),
    ('./../g', BASE_URI, 'http://a/b/g'),
    ('./g/.', BASE_URI, 'http://a/b/c/g/'),
    ('g/./h', BASE_URI, 'http://a/b/c/g/h'),
    ('g/../h', BASE_URI, 'http://a/b/c/h'),
    ('g;x=1/./y', BASE_URI, 'http://a/b/c/g;x=1/y'),
    ('g;x=1/../y', BASE_URI, 'http://a/b/c/y'),
    ('g?y/./x', BASE_URI, 'http://a/b/c/g?y/./x'),
    ('g?y/../x', BASE_URI, 'http://a/b/c/g?y/../x'),
    ('g#s/./x', BASE_URI, 'http://a/b/c/g#s/./x'),
    ('g#s/../x', BASE_URI, 'http://a/b/c/g#s/../x'),
    ('http:g', BASE_URI, ('http:g', 'http://a/b/c/g')),
    ]

# Test cases for URI reference syntax;
# some of these URIs come from a Perl script by David A. Wheeler
# at http://www.dwheeler.com/secure-programs/url.pl
#
good_URI_references = [
    'http://www.yahoo.com',
    'http://www.yahoo.com/',
    'http://1.2.3.4/',
    'http://www.yahoo.com/stuff',
    'http://www.yahoo.com/stuff/',
    'http://www.yahoo.com/hello%20world/',
    'http://www.yahoo.com?name=obi',
    'http://www.yahoo.com?name=obi+wan&status=jedi',
    'http://www.yahoo.com?onery',
    'http://www.yahoo.com#bottom',
    'http://www.yahoo.com/yelp.html#bottom',
    'https://www.yahoo.com/',
    'ftp://www.yahoo.com/',
    'ftp://www.yahoo.com/hello',
    'demo.txt',
    'demo/hello.txt',
    'demo/hello.txt?query=hello#fragment',
    '/cgi-bin/query?query=hello#fragment',
    '/demo.txt',
    '/hello/demo.txt',
    'hello/demo.txt',
    '/',
    '',
    '#',
    '#here',
    'http://www.yahoo.com?name=%00%01',
    'http://www.yaho%6f.com',
    'http://www.yahoo.com/hello%00world/',
    'http://www.yahoo.com/hello+world/',
    'http://www.yahoo.com?name=obi&',
    'http://www.yahoo.com?name=obi&type=',
    'http://www.yahoo.com/yelp.html#',
    '//',
    'file:///foo/bar',
    'mailto:user@host?subject=blah',
    ]

bad_URI_references = [
    'http://www yahoo.com',
    'http://www.yahoo.com/hello world/',
    'http://www.yahoo.com/yelp.html#"',
    '::',
    'beepbeep\x07\x07',
    '\n',
    ]

def Test(tester):

    tester.startGroup('UriDict')
    tester.startTest('file:/// and file://localhost/ identical as per RFC 1738')
    uris = Uri.UriDict()
    uris['file:///path/to/resource'] = 1
    uris['file://localhost/path/to/resource'] = 2
    tester.compare(2, uris['file:///path/to/resource'], 'RFC 1738 localhost support failed')
    tester.testDone()
    tester.groupDone()

    tester.startGroup('BaseJoin')
    for base, relative, expectedUri in basejoin_test_cases:
        tester.startTest('base=%r rel=%r' % (base,relative))
        res = Uri.BaseJoin(base, relative)
        tester.compare(expectedUri, res, 'Invalid result')
        tester.testDone()
    tester.groupDone()

    tester.startGroup('absolutize')
    for uriRef, baseUri, expectedUri in absolutize_test_cases:
        tester.startTest('doc=%r base=%r ref=%r' % (CURRENT_DOC_URI, baseUri, uriRef))
        res = Uri.Absolutize(uriRef, CURRENT_DOC_URI, baseUri)
        # in a couple cases, there's more than one correct result
        if type(expectedUri) == types.TupleType:
            tester.compare(1, res in expectedUri, 'Invalid result')
        else:
            if expectedUri.startswith(CURRENT_DOC_URI):
                expectedUri = None
            tester.compare(expectedUri, res, 'Invalid result')
        tester.testDone()
    tester.groupDone()

    tester.startGroup("URI reference syntax")
    for testuri in good_URI_references:
        tester.startTest("Good URI: %s" % repr(testuri))
        tester.compare(1, Uri.MatchesUriRefSyntax(testuri), "Fails to validate")
        tester.testDone()
    for testuri in bad_URI_references:
        tester.startTest("Bad URI: %s" % repr(testuri))
        tester.compare(0, Uri.MatchesUriRefSyntax(testuri), "Validates in error")
        tester.testDone()
    tester.groupDone()

    tester.startGroup("Basic Uri Resolver")
    data = [('http://foo.com/root/','path','http://foo.com/root/path'),
            ('http://foo.com/root','path','http://foo.com/path'),
            ]
    for base,uri,exp in data:
        tester.startTest("normalize: %s %s" % (base,uri))
        res = Uri.BASIC_RESOLVER.normalize(uri,base)
        tester.compare(exp,res)
        tester.testDone()

    base = 'foo:foo.com'
    uri = 'path'
    tester.startTest("normalize: %s %s" % (base,uri))
    tester.testException(Uri.BASIC_RESOLVER.normalize,(uri,base),Uri.UriException)
    tester.testDone()

    tester.startTest('resolve')
    base = os.getcwd()
    if base[-1] != os.sep:
        base += os.sep
    stream = Uri.BASIC_RESOLVER.resolve('test.py', Uri.OsPathToUri(base))
    tester.compare("#!/usr/bin/env python" + os.linesep, stream.readline())
    stream.close()
    tester.testDone()

    tester.startTest('generate')
    uuid = Uri.BASIC_RESOLVER.generate()
    tester.compare('urn:uuid:',uuid[:9])

    tester.testDone()

    tester.groupDone()

    tester.startGroup("SchemeRegistryResolver")

    def evalSchemeHandler(uri, base=None):
        if base: uri = base+uri
        uri = uri[5:]
        return str(eval(uri))

    def shiftSchemeHandler(uri, base=None):
        if base: uri = base+uri
        uri = uri[6:]
        return ''.join([ chr(ord(c)+1) for c in uri])

    resolver = Uri.SchemeRegistryResolver({'eval': evalSchemeHandler,
                                           'shift': shiftSchemeHandler,
                                           })

    scheme_cases = [(None, 'eval:150-50', '100'),
            (None, 'shift:abcde', 'bcdef'),
            ('eval:150-', '50', '100'),
            ('shift:ab', 'cde', 'bcdef'),
        ]

    for base, relative, expected in scheme_cases:
        tester.startTest("URI: base=%s uri=%s" % (base, relative))

        res = resolver.resolve(relative, base)
        tester.compare(expected, res)

        tester.testDone()

    resolver.handlers[None] = shiftSchemeHandler
    del resolver.handlers['shift']

    for base, relative, expected in scheme_cases:
        tester.startTest("URI: base=%s uri=%s" % (base, relative))

        res = resolver.resolve(relative, base)
        tester.compare(expected, res)

        tester.testDone()

    tester.groupDone()

    return


if __name__ == '__main__':
    from Ft.Lib.TestSuite import Tester
    tester = Tester.Tester()
    Test(tester)

