import os
from cStringIO import StringIO
from Xml.Xslt import test_harness
from Ft.Lib import Uri
from Ft.Xml import __version__
from Ft.Xml.InputSource import DefaultFactory
from Ft.Xml.Lib import TreeCompare

# See XSLT 1.0 spec section 12.
# These are the functions that should be tested:
#  document()
#  key() and xsl:key
#  format-number() and xsl:decimal-format
#  current()
#  unparsed-entity-uri()
#  generate-id()
#  system-property(), including these required properties:
#    system-property('xsl:version')
#    system-property('xsl:vendor')
#    system-property('xsl:vendor-url')

base = os.getcwd()
if base[-1] != os.sep:
    base += os.sep
base = Uri.OsPathToUri(base)


#----------------------------------------------------------------------
#
# system-property()
#
xml_source1 = """<dummy/>"""

sheet_str1 = """<?xml version="1.0"?>
<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:ft="http://xmlns.4suite.org/ext"
  xmlns:env="http://xmlns.4suite.org/xslt/env-system-property"
  version="1.0"
>
  <xsl:template match="/">
    <xsl:value-of select="system-property('xsl:vendor')"/><xsl:text>&#10;</xsl:text>
    <xsl:value-of select="system-property('xsl:vendor-url')"/><xsl:text>&#10;</xsl:text>
    <xsl:value-of select="system-property('xsl:version')"/><xsl:text>&#10;</xsl:text>
    <xsl:value-of select="system-property('ft:version')"/><xsl:text>&#10;</xsl:text>
    <xsl:value-of select="system-property('env:PATH')"/><xsl:text>&#10;</xsl:text>
  </xsl:template>

</xsl:transform>"""

expected1 = """<?xml version="1.0" encoding="UTF-8"?>
Fourthought Inc.
http://4Suite.org
1
%s
%s
"""%(__version__, os.environ.get('PATH', ''))


#----------------------------------------------------------------------
#
# format-number() and xsl:decimal-format
#
xml_source2 = xml_source1

sheet_str2 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" indent="yes" encoding="us-ascii"/>

  <xsl:decimal-format name="european" decimal-separator=',' grouping-separator='.' />
  <xsl:decimal-format name="underscore-as-minus" minus-sign='_' />
  <xsl:decimal-format name="d-digit" digit="D" />
  <xsl:decimal-format name="carat-percent" percent="^" />
  <xsl:decimal-format name="carat-permille" per-mille="^" />
  <xsl:decimal-format name="arabic-indic-digits" zero-digit='&#x0660;' />
  <xsl:decimal-format name="vertbar-patternsep" pattern-separator="|" />

  <xsl:template match="/">
    <result>
      <!-- 5,351 -->
      <r><xsl:value-of select="format-number(5351,'#,###')"/></r>
      <!-- 5351.00 -->
      <r><xsl:value-of select="format-number(5351,'#.00')"/></r>
      <!-- 53.5100 -->
      <r><xsl:value-of select="format-number(53.51,'#.0000')"/></r>
      <!-- 0053.5100 -->
      <r><xsl:value-of select="format-number(53.51,'0000.0000')"/></r>
      <!-- 0053.51 -->
      <r><xsl:value-of select="format-number(53.51,'0000.####')"/></r>
      <!-- 53.6 -->
      <r><xsl:value-of select="format-number(53.56,'0.0')"/></r>
      <!-- 24.535,20 -->
      <r><xsl:value-of select="format-number(24535.2, '###.###,00', 'european')"/></r>
      <!-- NaN -->
      <r><xsl:value-of select="format-number('abc', '#,###')"/></r>
      <r><xsl:value-of select="format-number('', '#,###')"/></r>
      <!-- Infinity -->
      <r><xsl:value-of select="format-number(2 div 0, '#,###')"/></r>
      <!-- -5,351 -->
      <r><xsl:value-of select="format-number(-5351, '#,###')"/></r>
      <!-- -5,351 -->
      <r><xsl:value-of select="format-number(-5351, '#,###', 'underscore-as-minus')"/></r>
      <!-- (5,351) -->
      <r><xsl:value-of select="format-number(-5351,'###,###,###;(###,###,###)')"/></r>
      <!-- +5,351 -->
      <r><xsl:value-of select="format-number(5351,'+###,###,###;-###,###,###')"/></r>
      <!-- -5,351 -->
      <r><xsl:value-of select="format-number(-5351,'+###,###,###|-###,###,###','vertbar-patternsep')"/></r>
      <!-- 5,351 -->
      <r><xsl:value-of select="format-number(5351,'D,DDD', 'd-digit')"/></r>
      <!-- 12% -->
      <r><xsl:value-of select="format-number(0.12,'#00%')"/></r>
      <!-- 12^ -->
      <r><xsl:value-of select="format-number(0.115,'#00^','carat-percent')"/></r>
      <!-- 015&#x2030; or 015&#8240; -->
      <r><xsl:value-of select="format-number(0.015,'000&#x2030;')"/></r>
      <!-- 015^ -->
      <r><xsl:value-of select="format-number(0.015,'000^','carat-permille')"/></r>
      <!-- $12,345 -->
      <r><xsl:value-of select="format-number(12345,'$#0,000')"/></r>
      <!-- $12,345.00 -->
      <r><xsl:value-of select="format-number(12345,'$##,000.00')"/></r>
      <!-- $0,123.45 -->
      <r><xsl:value-of select="format-number(123.45,'$#0,000.00')"/></r>
      <!-- 012 -->
      <r><xsl:value-of select="format-number(12,'000')"/></r>
      <!-- &#x283C;&#x281A;&#x2801;&#x2803; or &#10300;&#10266;&#10241;&#10243; -->
      <r>012 in Braille: <xsl:value-of select="translate(format-number(12,'&#x283C;000'),'012345679','&#x281A;&#x2801;&#x2803;&#x2809;&#x2819;&#x2811;&#x280B;&#x281B;&#2813;&#x280A;')"/></r>
      <!-- &#x0660;&#x0661;&#x0662; or &#1632;&#1633;&#1634; -->
      <r>012 in Arabic-Indic digits: <xsl:value-of select="format-number(12,'&#x660;&#x660;&#x660;','arabic-indic-digits')"/></r>
    </result>
  </xsl:template>

</xsl:stylesheet>"""

expected2 = """<?xml version="1.0" encoding="us-ascii"?>
<result>
  <r>5,351</r>
  <r>5351.00</r>
  <r>53.5100</r>
  <r>0053.5100</r>
  <r>0053.51</r>
  <r>53.6</r>
  <r>24.535,20</r>
  <r>NaN</r>
  <r>NaN</r>
  <r>Infinity</r>
  <r>-5,351</r>
  <r>_5,351</r>
  <r>(5,351)</r>
  <r>+5,351</r>
  <r>-5,351</r>
  <r>5,351</r>
  <r>12%</r>
  <r>12^</r>
  <r>015&#8240;</r>
  <r>015^</r>
  <r>$12,345</r>
  <r>$12,345.00</r>
  <r>$0,123.45</r>
  <r>012</r>
  <r>012 in Braille: &#10300;&#10266;&#10241;&#10243;</r>
  <r>012 in Arabic-Indic digits: &#1632;&#1633;&#1634;</r>
</result>"""


#----------------------------------------------------------------------
#
# current()
#
xml_source3 = """<?xml version="1.0"?>
<doc>
  <elem a1="1" a2="6">one</elem>
  <elem a1="2" a2="4">two</elem>
  <elem a1="3" a2="2">three</elem>
</doc>"""

sheet_str3 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <result>
      <xsl:apply-templates/>
    </result>
  </xsl:template>

  <xsl:template match="doc">
    <xsl:apply-templates select="elem">
      <xsl:sort select="@a1" order="descending"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="elem">
    <r>
      <xsl:value-of select="current()"/>
    </r>
  </xsl:template>

</xsl:stylesheet>"""

expected3 = """<?xml version="1.0" encoding="UTF-8"?>
<result>
  <r>three</r>
  <r>two</r>
  <r>one</r>
</result>"""


xml_source4 = xml_source3

sheet_str4 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <result>
      <xsl:apply-templates/>
    </result>
  </xsl:template>

  <xsl:template match="doc">
    <xsl:apply-templates select="elem">
      <xsl:sort select="@a1" order="descending"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="elem">
    <r1>
      <xsl:value-of select="current()"/>
    </r1>
    <r2>
      <xsl:value-of select="/doc/elem[@a2=current()/@a1*2]"/>
    </r2>
  </xsl:template>

</xsl:stylesheet>"""

expected4 = """<?xml version="1.0" encoding="UTF-8"?>
<result>
  <r1>three</r1>
  <r2>one</r2>
  <r1>two</r1>
  <r2>two</r2>
  <r1>one</r1>
  <r2>three</r2>
</result>"""


#----------------------------------------------------------------------
#
# document() only
#
xml_source5 = xml_source1

sheet_str5 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:vdv="http://eric.van-der-vlist.com/tmpns" version="1.0">
 <xsl:output method="xml" encoding="utf-8"/>
 <xsl:template match="/">
  <xsl:copy-of select="document('')"/>
 </xsl:template>
</xsl:transform>"""

expected5 = sheet_str5


xml_source6 = xml_source1

sheet_str6 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
 <xsl:output method="xml" encoding="utf-8"/>
 <xsl:template match="/">
  <xsl:copy-of select="document('Xml/Xslt/Core/addr_book1.xml')"/>
 </xsl:template>
</xsl:stylesheet>"""

sheet_uri6 = base + 'sheet6.xsl'

f = open(os.path.normpath('Xml/Xslt/Core/addr_book1.xml'), 'rb')
expected6 = f.read()
f.close()


#----------------------------------------------------------------------
#
# generate-id() only
#
xml_source7 = xml_source4

sheet_str7 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exsl="http://exslt.org/common">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <result>
      <!-- make sure multiple calls and order of calls don't matter -->
      <xsl:variable name="all-ids1">
        <xsl:for-each select="//node()">
          <id val="{.}"><xsl:value-of select="generate-id()"/></id>
        </xsl:for-each>
      </xsl:variable>
      <xsl:variable name="all-ids2">
        <xsl:for-each select="//node()">
          <xsl:sort select="." order="descending"/>
          <id val="{.}"><xsl:value-of select="generate-id()"/></id>
        </xsl:for-each>
      </xsl:variable>
      <!-- != means at least one string-value in first set is different than a string-value in the other. -->
      <!-- assumes exsl:node-set() and node-set comparisons work properly! -->
      <r>IDs are unique: <xsl:value-of select="not(exsl:node-set($all-ids1)/ids1 != exsl:node-set($all-ids2)/ids2)"/></r>
      <xsl:variable name="invalid-id">
        <xsl:for-each select="//node()">
          <xsl:if test="translate(substring(generate-id(),1,1),'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ','')">!</xsl:if>
          <xsl:if test="translate(substring(generate-id(),2),'0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ','')">!</xsl:if>
        </xsl:for-each>
      </xsl:variable>
      <r>IDs are valid: <xsl:value-of select="boolean($invalid-id)"/></r>
      <r>IDs for context node don't vary: <xsl:value-of select="generate-id()=generate-id(/)=generate-id(.)"/></r>
      <r>ID for empty node-set is empty string: <xsl:value-of select="generate-id(/..)=''"/></r>
    </result>
  </xsl:template>

</xsl:stylesheet>"""

expected7 = """<?xml version='1.0' encoding='UTF-8'?>
<result xmlns:exsl='http://exslt.org/common'>
  <r>IDs are unique: true</r>
  <r>IDs are valid: true</r>
  <r>IDs for context node don't vary: true</r>
  <r>ID for empty node-set is empty string: true</r>
</result>"""


#----------------------------------------------------------------------
#
# generate-id() and document()
#
xml_source8 = xml_source4

sheet_str8 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exsl="http://exslt.org/common">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <result>
      <r>stylesheet root is unique: <xsl:value-of select="generate-id(document(''))=generate-id(document(''))"/></r>
      <r>external document root is unique: <xsl:value-of select="generate-id(document('Xml/Xslt/Core/addr_book1.xml'))=generate-id(document('Xml/Xslt/Core/addr_book1.xml'))"/></r>
      <r>stylesheet root != external non-stylesheet document root: <xsl:value-of select="generate-id(document('Xml/Xslt/Core/addr_book1.xml'))!=generate-id(document(''))"/></r>
    </result>
  </xsl:template>

</xsl:stylesheet>"""

sheet_uri8 = base + 'sheet8.xsl'

expected8 = """<?xml version='1.0' encoding='UTF-8'?>
<result xmlns:exsl='http://exslt.org/common'>
  <r>stylesheet root is unique: true</r>
  <r>external document root is unique: true</r>
  <r>stylesheet root != external non-stylesheet document root: true</r>
</result>"""

# This test relies on RFC 2396-conformant URI resolution.
# Equivalent URIs must resolve to the same node with the same ID;
# documents must not be treated as unique when their URIs are equivalent.

xml_source9 = xml_source1

source_uri9 = base + 'Xml/Xslt/Core/dummy.xml'
sheet_uri9 = base + 'Xml/Xslt/Core/genid.xsl'
sheet_params9 = {
    u'src-uri': source_uri9,
    u'sty-uri': sheet_uri9
}

expected9 = """<?xml version='1.0' encoding='us-ascii'?>
<results>
  <group>
    <title>source doc IDs</title>
    <test>ID of root node of source via XPath location path / is non-empty string</test>
    <result>true</result>
    <test>ID of root node of source via same-document URI reference relative to XPath location path / does not differ</test>
    <result>true</result>
    <test>ID of root node of source via relative URI ref does not differ</test>
    <result>true</result>
    <test>ID of root node of source via absolute URI ref does not differ</test>
    <result>true</result>
    <test>all of the above IDs are identical</test>
    <result>true</result>
    <test>ID of root node of source via file://localhost/ absolute URI ref does not differ</test>
    <result>true</result>
  </group>
  <group>
    <title>stylesheet IDs</title>
    <test>ID of root node of stylesheet via same-document URI reference is non-empty string</test>
    <result>true</result>
    <test>above ID differs from ID of root node of source via XPath location path /</test>
    <result>true</result>
    <test>ID of root node of stylesheet via relative URI ref does not differ</test>
    <result>true</result>
    <test>ID of root node of stylesheet via absolute URI ref does not differ</test>
    <result>true</result>
    <test>all of the above IDs are identical</test>
    <result>true</result>
    <test>ID of root node of stylesheet via file://localhost/ absolute URI ref does not differ</test>
    <result>true</result>
  </group>
  <group>
    <title>factors affecting results above</title>
    <test>generate-id() on empty node-set returns empty string</test>
    <result>true</result>
    <test>generate-id() on same node returns same results</test>
    <result>true</result>
    <test>generate-id() on different nodes returns different results</test>
    <result>true</result>
    <test>URIs given as src-uri and sty-uri are resolvable</test>
    <result>true</result>
    <test>given same URI reference, document() returns same node</test>
    <result>true</result>
    <test>given equivalent relative and absolute URI references, document() returns same node</test>
    <result>true</result>
    <test>URI given as src-uri parameter is recognized as URI of source doc fed to processor</test>
    <result>true</result>
    <test>URI given as sty-uri parameter is recognized as URI of stylesheet fed to processor</test>
    <result>true</result>
    <test>URI resolver treats file:/// and file://localhost/ same as per RFC 1738</test>
    <result>true</result>
  </group>
</results>"""

#----------------------------------------------------------------------
#
# unparsed-entity-uri()
#
xml_source10 = """<?xml version='1.0' encoding='UTF-8'?>
<!DOCTYPE foo [
  <!NOTATION png SYSTEM "http://www.libpng.org/pub/png/">
  <!ENTITY obj0 SYSTEM "http://www.w3.org/Icons/WWW/w3c_home" NDATA png>
  <!ELEMENT foo (bar)>
  <!ELEMENT bar (#PCDATA)>
  <!ATTLIST bar
     logo ENTITY #IMPLIED
     desc CDATA #REQUIRED>
]>
<foo>
  <bar logo="obj0" desc="W3C logo"/>
</foo>"""

sheet_str10 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <result>
      <xsl:value-of select="unparsed-entity-uri('obj0')"/>
    </result>
  </xsl:template>

</xsl:stylesheet>"""

expected10 = """<?xml version='1.0' encoding='UTF-8'?>
<result>http://www.w3.org/Icons/WWW/w3c_home</result>"""

xml_source11 = xml_source1

sheet_str11 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
 <xsl:output method="xml" encoding="utf-8"/>
 <xsl:template match="/">
  <xsl:copy-of select="document('urn:fourthought:Xml/Xslt/Core/addr_book1.xml')"/>
 </xsl:template>
</xsl:stylesheet>"""

sheet_uri11 = base + 'sheet11.xsl'

expected11 = expected6

# URN resolver class used for test11
class URNResolver(Uri.SchemeRegistryResolver):
    def __init__(self):
        Uri.SchemeRegistryResolver.__init__(self)
        self.handlers['urn'] = self.resolveURN
        self.supportedSchemes.append('urn')

    def resolveURN(self, uri, base=None):
        uriParts = uri.split(':')
        nid      = uriParts[1] # namespace ID
        nss      = uriParts[2] # namespace specific string
        f = open(os.path.normpath(nss), 'rb')
        st = f.read()
        f.close()
        return StringIO(st)

def Test(tester):

    source = test_harness.FileInfo(string=xml_source1)
    sheet = test_harness.FileInfo(string=sheet_str1)
    test_harness.XsltTest(tester, source, [sheet], expected1, compareFunc=cmp,
                          title='system-property()')

    source = test_harness.FileInfo(string=xml_source2)
    sheet = test_harness.FileInfo(string=sheet_str2)
    test_harness.XsltTest(tester, source, [sheet], expected2,
                          compareFunc=TreeCompare.TreeCompare,
                          title='format-number() and xsl:decimal-format')

    source = test_harness.FileInfo(string=xml_source3)
    sheet = test_harness.FileInfo(string=sheet_str3)
    test_harness.XsltTest(tester, source, [sheet], expected3,
                          compareFunc=TreeCompare.TreeCompare,
                          title='current() vs XSLT context')

    source = test_harness.FileInfo(string=xml_source4)
    sheet = test_harness.FileInfo(string=sheet_str4)
    test_harness.XsltTest(tester, source, [sheet], expected4,
                          compareFunc=TreeCompare.TreeCompare,
                          title='current() vs XPath predicate context')

    source = test_harness.FileInfo(string=xml_source5)
    sheet = test_harness.FileInfo(string=sheet_str5)
    test_harness.XsltTest(tester, source, [sheet], expected5,
                          compareFunc=TreeCompare.TreeCompare,
                          title='document() only: stylesheet')

    source = test_harness.FileInfo(string=xml_source6)
    sheet = test_harness.FileInfo(string=sheet_str6, baseUri=sheet_uri6)
    test_harness.XsltTest(tester, source, [sheet], expected6,
                          compareFunc=TreeCompare.TreeCompare,
                          title='document() only: external XML file')

    source = test_harness.FileInfo(string=xml_source7)
    sheet = test_harness.FileInfo(string=sheet_str7)
    test_harness.XsltTest(tester, source, [sheet], expected7,
                          compareFunc=TreeCompare.TreeCompare,
                          title='generate-id() only')

    source = test_harness.FileInfo(string=xml_source8)
    sheet = test_harness.FileInfo(string=sheet_str8, baseUri=sheet_uri8)
    test_harness.XsltTest(tester, source, [sheet], expected8,
                          compareFunc=TreeCompare.TreeCompare,
                          title='generate-id() and document(): ID uniqueness')

    source = test_harness.FileInfo(string=xml_source9, baseUri=source_uri9)
    sheet = test_harness.FileInfo(uri=sheet_uri9)
    test_harness.XsltTest(tester, source, [sheet], expected9,
                          compareFunc=TreeCompare.TreeCompare,
                          topLevelParams=sheet_params9,
                          title='generate-id() and document(): ID consistency')

    source = test_harness.FileInfo(string=xml_source10, validate=1)
    sheet = test_harness.FileInfo(string=sheet_str10)
    test_harness.XsltTest(tester, source, [sheet], expected10,
                          title='unparsed-entity-uri()')

    source = test_harness.FileInfo(string=xml_source11)
    sheet = test_harness.FileInfo(string=sheet_str11, baseUri=sheet_uri11)
    originalResolver = DefaultFactory.resolver
    DefaultFactory.resolver = URNResolver()
    test_harness.XsltTest(tester, source, [sheet], expected11,
                          compareFunc=TreeCompare.TreeCompare,
                          title='document() resolve URN via SchemeRegistryResolver')
    DefaultFactory.resolver = originalResolver

    return
