import os
from Ft.Lib import Uri, Uuid
from Ft.Xml import FtMiniDom, InputSource, Domlette, cDomlette
from Ft.Xml.Lib import TreeCompare
from Ft.Xml.Xslt import Processor, XsltException, Error

# Needed to have comparable id's for test results
from Ft.Xml.Xslt import XsltFunctions
g_idmap = {}
def GenerateId(context, nodeSet=None):
    gen_id = XsltFunctions.GenerateId(context, nodeSet)
    if gen_id:
        id = g_idmap.setdefault(gen_id, len(g_idmap)+1)
        gen_id = u'id%d' % id
    return gen_id

ExtFunctions = {(None, 'generate-id') : GenerateId}


def GetReader(tester):
    source = tester.test_data['source']
    if source == 'cDomlette':
        reader = Domlette._Reader(cDomlette.nonvalParse)
    elif source == 'minidom':
        reader = Domlette._Reader(FtMiniDom.nonvalParse)
    else:
        raise ValueError('unknown source %r' % source)
    return reader


class FileInfo:
    def __init__(self, uri=None, string=None, baseUri='', validate=0):
        self.uri = uri
        self.string = string
        self.baseUri = baseUri
        self.validate = validate

        # Same logic that exists in _4xslt.py
        # The processor only deals with URIs, not file paths
        if uri and os.path.isfile(uri):
            self.uri = Uri.OsPathToUri(os.path.abspath(uri))
        if not (uri or string):
            raise ValueError('Either string or uri are required')

        if string and not baseUri:
            base = os.path.join(os.path.abspath(os.getcwd()),
                                Uuid.UuidAsString(Uuid.GenerateUuid()))
            self.baseUri = Uri.OsPathToUri(base)
        return

    def toInputSource(self, factory):
        if self.uri:
            source = factory.fromUri(self.uri)
        else:
            source = factory.fromString(self.string, self.baseUri)
        return source


def XsltTest(tester, source, stylesheets, expected, compareFunc=None,
             ignorePis=1, topLevelParams=None, extensionModules=None,
             exceptionCode=None, stylesheetInputFactory=None,
             documentInputFactory=None, stylesheetIncPaths=None,
             title=''):

    # reset the counter for generate-id
    g_idmap.clear()

    if not title:
        title = 'Source %(source)s' % (tester.test_data)
    tester.startTest(title)

    # Create the processor
    processor = Processor.Processor(stylesheetIncPaths)
    processor.registerExtensionModules([__name__])
    if extensionModules:
        processor.registerExtensionModules(extensionModules)

    # Setup document input for the processor
    factory = documentInputFactory or InputSource.DefaultFactory
    reader = GetReader(tester)
    reader.inputSourceFactory = factory
    processor.setDocumentReader(reader)

    # Do the transformation
    try:
        # Add the stylesheets to the processor
        factory = stylesheetInputFactory or InputSource.DefaultFactory
        for stylesheet in stylesheets:
            processor.appendStylesheet(stylesheet.toInputSource(factory))

        result = processor.run(source.toInputSource(factory),
                               ignorePis=ignorePis,
                               topLevelParams=topLevelParams)
    except XsltException, exception:
        if exceptionCode is None:
            # We wanted to compare the processor result
            tester.exception('Unexpected XsltException raised')
        else:
            # Compare the exception result
            tester.compare(exceptionCode, exception.errorCode, stackLevel=2)
    else:
        if exceptionCode is not None:
            # We wanted to compare the exception result
            tester.error("XsltException(%d) not raised" % exceptionCode,
                         stackLevel=2)
        else:
            # Compare the processor result
            tester.compare(expected, result, diff=1,
                           func=compareFunc or TreeCompare.TreeCompare,
                           stackLevel=2)

    tester.testDone()
    return
