from xml.dom import Node
from Ft.Xml import XML_NAMESPACE, XMLNS_NAMESPACE, EMPTY_PREFIX

def PyGetAllNs(node):
    #The xml namespace is implicit
    nss = {'xml': XML_NAMESPACE}
    if node.nodeType == Node.ATTRIBUTE_NODE and node.ownerElement:
        return PyGetAllNs(node.ownerElement)
    if node.nodeType == Node.ELEMENT_NODE:
        # There are some edge cases here to worry about.
        #1. The element's prefix/ns could be different then a ns decl, the element should win
        #2  An Attributes preifx/ns could be different then a ns decl or the elements.  The element should win
        #   over all and the attributes should win over ns decls

        for attr in node.attributes.values():
            # xmlns="uri" or xmlns:foo="uri"
            if attr.namespaceURI == XMLNS_NAMESPACE:
                if attr.localName == 'xmlns':
                    #Not swapped around
                    if not nss.has_key(attr.prefix):
                        nss[attr.prefix] = attr.value
                # xmlns:foo="uri"
                elif attr.prefix:
                    # "foo" prefix -> uri
                    if not nss.has_key(attr.localName):
                        nss[attr.localName] = attr.value
                # xmlns="uri"
                else:
                    # default ns -> uri
                    if not nss.has_key(EMPTY_PREFIX):
                        nss[EMPTY_PREFIX] = attr.value
            # A prefixed attribute name may have had its namespace URI set
            # by the DOM without there having been an xmlns:prefix="uri"
            # that would have been detected by the code above
            elif attr.namespaceURI:
                nss[attr.prefix] = attr.namespaceURI
        if node.namespaceURI:
            nss[node.prefix] = node.namespaceURI

    if node.parentNode:
        #Inner NS/Prefix mappings take precedence over outer ones
        parent_nss = PyGetAllNs(node.parentNode)
        parent_nss.update(nss)
        nss = parent_nss
    return nss

try:
    from Ft.Xml import cDomlettec
    GetAllNs = cDomlettec.GetAllNs
except ImportError:
    GetAllNs = PyGetAllNs

def SeekNss(node, nss=None):
    '''traverses the tree to seek an approximate set of defined namespaces'''
    if nss is None:
        nss = {}
    for child in node.childNodes:
        if child.nodeType == Node.ELEMENT_NODE:
            if child.namespaceURI:
                nss[child.prefix] = child.namespaceURI
            for attr in child.attributes.values():
                # xmlns="uri" or xmlns:foo="uri"
                if attr.namespaceURI == XMLNS_NAMESPACE:
                    # xmlns:foo="uri"
                    if attr.localName == 'xmlns':
                        #Not swapped around
                        nss[attr.prefix] = attr.value
                    elif attr.prefix:
                        # "foo" prefix -> uri
                        nss[attr.localName] = attr.value
                    # xmlns="uri"
                    else:
                        # default ns -> uri
                        nss[EMPTY_PREFIX] = attr.value
                # A prefixed attribute name may have had its namespace URI set
                # by the DOM without there having been an xmlns:prefix="uri"
                # that would have been detected by the code above
                elif attr.namespaceURI:
                    nss[attr.prefix] = attr.namespaceURI
            SeekNss(child, nss)
    return nss
